!This routine computes the necessary boundary conditions
! at the nozzle exit and entrance.  For a subsonic inlet,
! pressure is simply extrapolated backwards from the
! upstream.  Supersonic inlet pressure is computed
! on the basis of given mach number.
! A supersonic outlet extrapolates the pressure, while
! a subsonic inlet sets the back pressure to the value
! specified by the user.




      Subroutine Bound(V,p,min,pback,lmax,mx,my,exit)
      Integer lmax,mx,my
      Real V(lmax,mx+1,my+1)
      Real  p(mx+1,my+1)
      Real mach(my+1)
      Real min,pback
      Integer exit1
! Simple leave incoming distribution and alignment alone
!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto * PROC
!HPF$ ALIGN (*,:) with *V (*,*,:) :: p

  
      If (min .gt. 1.0) then
       Forall(j=1:my+1)
        P(1,j)=1.0/((min**2)/2.0 *(1.4-1.0)
     $  +1.0)**(1.4/0.4)
        V(1,1,j)=P(1,j)*(1.0
     $  + (1.4-1.0)/2.0*min**2)
        V(2,1,j)=V(1,1,j)*min*
     $ sqrt(1.4*p(1,j)/V(1,1,j))
        V(3,1,j)=0.0
        V(4,1,j)=p(1,j)/(0.4) + 0.5*V(1,1,j)
     $ *(V(2,1,j)**2 / V(1,1,j)**2)
        end Forall
       End IF

       If (min .le. 1.0) then
        Forall(j=1:my+1)
        p(1,j)=2.0*p(2,j)-p(3,j)
    	 mach(j)=2.0/(1.4-1.0)*(p(1,j)**
     $ ((1.0-1.4)/1.4) -1.0)
         V(1,1,j)=p(1,j)*
     $ (1.0+0.2*mach(j))
        V(2,1,j)=sqrt(mach(j))*sqrt(1.4/(1.0+0.2*mach(j)))
     $ *V(1,1,j)

        V(3,1,j)=0.0

        V(4,1,j)=p(1,j)/(0.4) + 0.5*V(1,1,j)
     $ *(V(2,1,j)**2 / V(1,1,j)**2)
        end Forall
       End IF

       If (exit1 .eq. 1) then
          Forall(j=1:my+1)
          p(mx+1,j)=pback
           V(1,mx+1,j)=2.0*V(1,mx,j)-V(1,mx-1,j)
           V(2,mx+1,j)=2.0*V(2,mx,j)-V(2,mx-1,j)
           V(3,mx+1,j)=2.0*V(3,mx,j)-V(3,mx-1,j)
           V(4,mx+1,j)=pback/0.4
     $ + 0.5*V(1,mx+1,j)*(V(2,mx+1,j)**2/v(1,mx+1,j)**2 +
     $ V(3,mx+1,j)**2/V(1,mx+1,j)**2)
           end Forall
        End if

        If (exit1 .eq. 2 ) then
           Forall(j=1:my+1)
           p(mx+1,j) =p(mx,j)

            V(1,mx+1,j)=V(1,mx,j)

           V(2,mx+1,j)=V(2,mx,j)

           V(3,mx+1,j)=V(3,mx,j)
    	  
           V(4,mx+1,j)=p(mx+1,j)/0.4
     $ + 0.5*V(1,mx+1,j)*(V(2,mx+1,j)**2/v(1,mx+1,j)**2 +
     $ V(3,mx+1,j)**2/V(1,mx+1,j)**2)
           end Forall
        End if


        End subroutine







