! This routine computes the arificial dissipation for the scheme.
! The dissipation vectors avab,avbc,avcd,acda are necessary to
! dampen the numerical oscillations that a shock wave produces.
! The vectors are the sum of a second order term and fourth
! order term.  The fourth order term is turned "off" when
! a pressure gradient sensor EPS2 reaches a critical value (kp4).
! Leaving it in would blow the solution up near shock waves.

! Two HPF_LOCAL routines are used because the boundary conditions
! the routine requires (Derivatives are zero at boundaries)
! somehow required communication when they shouldn't have.
! Using HPF_LOCAL insures the procedure is done on a single
! node with no communication.



    	Extrinsic (hpf_local) subroutine any(V,lmax,mx,my)
    	integer lmax,mx,my
        Real   V(:,:,:)
        integer myid,nprocs,ub,lb,i,j,k
!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto *PROC


    	myid   = pghpf_myprocnum()
    	nprocs = pghpf_nprocs()
! lb and ub are the upper and lower bound of the array's 3rd
! dimension

    	lb = lbound(V,3)
        ub = ubound(V,3)
        Do i=2,mx
!Make derivaties zero at top and lower boundary
           Do k=1,lmax
             if (myid == nprocs-1) then
       	       V(k,i,ub)=V(k,i,ub-1)
             else if (myid == 0) then
    	       V(k,i,lb)=V(k,i,lb+1)
             endif
           End Do
        End Do
       end subroutine

    	
    	extrinsic (hpf_local) subroutine notany(V,p,lmax,mx,my)
        integer mx,my,lmax
    	Real V(:,:,:)
        Real   p(:,:)
        integer myid,nprocs,ub,lb,i,j,k
!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto *PROC
!HPF$ ALIGN (*,:) with *V(*,*,:) :: p
        myid   = pghpf_myprocnum()
        nprocs = pghpf_nprocs()
 
        lb = lbound(p,2)
        ub = ubound(p,2)
! Make derivatives zero at top and lower boundary
        Do i=2,mx
          
             if (myid == nprocs-1) then
               p(i,ub)=p(i,ub-1)
             else if (myid == 0) then
               p(i,lb)=p(i,lb+1)
             endif
            End Do
       end subroutine


      Subroutine Artvisc(V,avab,avbc,avcd,avda,xloc,yloc,
     $ dnax,dnay,dnbx,dnby,p,lmax,mx,my)
      ! Implicit none
      Integer lmax,mx,my
      Real   V(lmax,mx+1,my+1)
      Real   avab(lmax,mx+1,my+1),avbc(lmax,mx+1,my+1),
     $ avcd(lmax,mx+1,my+1),avda(lmax,mx+1,my+1)
      Real  xloc(mx,my+1),yloc(mx,my+1),dnbx(mx,my+1),dnby(mx,my+1)
      Real   dnax(mx,my+1),dnay(mx,my+1)
      Real  p(mx+1,my+1)
      Real eps2(mx+1,my+1),eps4(mx+1,my+1)
      Real kp2,kp4,start,end
      Integer k, i,j,imy

! Execpt for eps2 and eps4, all arrays have been previously
! distributed and aligned.  Again, the * tells
! the compiler not to remap or realign.


!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto *PROC
!HPF$ ALIGN (*,:) with *V(*,*,:) :: xloc,yloc,dnax,dnay,dnbx,dnby
!HPF$ ALIGN (*,*,:) with *V (*,*,:) ::avab,avbc,avcd,avda
!HPF$ ALIGN (*,:) with *V(*,*,:) :: p
!HPF$ ALIGN  (*,:) with V(*,*,:) :: eps2,eps4 
 
!! HPF_Local routines must be interfaced, and they
! must have assumed shaped (:,:,:) array declarations
! They also use same mapping.
    	interface
    	extrinsic (hpf_local) subroutine any(V,lmax,mx,my)
    	integer lmax,mx,my
        Real   V(:,:,:)
        integer myid,nprocs,ub,lb,i,j,k
!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto *PROC
    	
    	end subroutine
        end interface

    	interface
    	extrinsic (hpf_local) subroutine notany(V,p,lmax,mx,my)
        integer mx,my,lmax
    	Real V(:,:,:)
        Real   p(:,:)
        integer myid,nprocs,ub,lb,i,j,k
!HPF$ PROCESSORS PROC(NUMBER_OF_PROCESSORS())
!HPF$ DISTRIBUTE  V *(*,*,BLOCK) onto *PROC
!HPF$ ALIGN (*,:) with *V(*,*,:) :: p
    	end subroutine
    	end interface

      kp2=4.0/4.0
      kp4=4.0/256.0

    	avab=0.0
    	avbc=0.0
    	avcd=0.0
    	avda=0.0
    	
    	


     
!***********Side AB & CD fluxes**********
        
!       call  any(V,lmax,mx,my)
    !	call notany(V,p,lmax,mx,my)

! What the  HPF_LOCAL routines do

    	p(:,my+1) = p(:,my)
      p(:,1) = p(:,2)
    	V(:,:,my+1)=V(:,:,my)
    	V(:,:,1)=V(:,:,2)
      Forall(k=1:lmax,i=2:mx)
    	 avab(k,i,2)=0.0
       avcd(k,i,my)=0.0
    	End Forall

! Instead of the Forall Constructs, I could have used the
! following Independent DO loops with scalar eps2 and eps4 variables

!  !HPF$  INDEPENDENT, new(i)
!       DO j=3,my
!  !HPF$ INDEPENDENT, new(k,eps2,eps4)
!         DO i=2,mx
!  !HPF$ INDEPENDENT 
    !	    DO k=1,4
    !	 eps2=abs(p(i,j+1)
!     $-2.0*p(i,j)+p(i,j-1))/abs(p(i,j+1)+2.0*p(i,j) +
!     $ p(i,j-1))
    !	eps2=kp2*max(eps2,abs(p(i,j)-2.0*p(i,j-1)+p(i,j-2))
!     $ /abs( p(i,j)+2.0*p(i,j-1)+p(i,j-2)))
!
    !	eps4=max(0.0,(kp4-eps2))

    !	 avab(k,i,j)=0.5*(V(2,i,j)/V(1,i,j)+V(2,i,j-1)/V(1,i,j-1))
!     $ *dnax(i,j)
! 
!        avab(k,i,j)=avab(k,i,j)+0.5*(V(3,i,j)/V(1,i,j)+
!     $V(3,i,j-1)/V(1,i,j-1))*
!     $ dnay(i,j)
!        avab(k,i,j)=abs(avab(k,i,j))
!        avab(k,i,j)=avab(k,i,j)/sqrt(dnax(i,j)**2 + dnay(i,j)**2)
!        avab(k,i,j)=avab(k,i,j)+sqrt(1.4*(p(i,j)+p(i,j-1))/
!     $ (V(1,i,j)+V(1,i,j-1)))
!        avab(k,i,j)=avab(k,i,j)*(eps2(i,j)*(V(k,i,j)-V(k,i,j-1))
!     $ -eps4(i,j)*(V(k,i,j+1)-V(k,i,j)-V(k,i,j)
    !     	    End DO
    !	  End DO
!       End DO 

      Forall(i=2:mx,j=3:my)

      eps2(i,j)=abs(p(i,j+1)
     $-2.0*p(i,j)+p(i,j-1))/abs(p(i,j+1)+2.0*p(i,j) +
     $ p(i,j-1))
    	eps2(I,J)=kp2*MAX(eps2(i,j),abs(p(i,j)-2.0*p(i,j-1)+p(i,j-2))
     $ /abs( p(i,j)+2.0*p(i,j-1)+p(i,j-2)))
      
       eps4(i,j)=max(0.0,(kp4-eps2(i,j)))
       End Forall
    	
    	
       Forall(k=1:lmax,i=2:mx,j=3:my)

          	avab(k,i,j)=0.5*(V(2,i,j)/V(1,i,j)+V(2,i,j-1)/V(1,i,j-1))
     $ *dnax(i,j)

    	avab(k,i,j)=avab(k,i,j)+0.5*(V(3,i,j)/V(1,i,j)+
     $V(3,i,j-1)/V(1,i,j-1))*
     $ dnay(i,j)
    	avab(k,i,j)=abs(avab(k,i,j))
    	avab(k,i,j)=avab(k,i,j)/sqrt(dnax(i,j)**2 + dnay(i,j)**2)
    	avab(k,i,j)=avab(k,i,j)+sqrt(1.4*(p(i,j)+p(i,j-1))/
     $ (V(1,i,j)+V(1,i,j-1)))
    	avab(k,i,j)=avab(k,i,j)*(eps2(i,j)*(V(k,i,j)-V(k,i,j-1))
     $ -eps4(i,j)*(V(k,i,j+1)-V(k,i,j)-V(k,i,j)-V(k,i,j)
     $ +V(k,i,j-1)+V(k,i,j-1)+V(k,i,j-1)-V(k,i,j-2)))
     
       End Forall 
    	
    

     

      Forall(k=1:lmax,i=2:mx,j=2:my-1) avcd(k,i,j)=1.0*avab(k,i,j+1)
      
!***********now BC and DA dissipation *********

      Forall(k=1:lmax,j=1:my+1)
       avda(k,2,j)=0.0
       avbc(k,mx,j)=0.0
    	avbc(k,mx+1,j)=0.0
       V(k,mx+1,j)=V(k,mx,j)
    	V(k,1,j)=V(k,2,j)
      End Forall

      Forall(j=1:my+1)
       p(mx+1,j)=p(mx,j)
    	p(1,j)=p(2,j)
      End Forall

          	
      Forall(i=2:mx-1,j=2:my)
      eps2(i,j)=abs(p(i+1,j)
     $ -2.0*p(i,j)+p(i-1,j))/abs(p(i+1,j)+2.0*p(i,j)+p(i-1,j))

    	eps2(i,j)=kp2*max(eps2(i,j),
     $ abs(p(i+2,j)-2.0*p(i+1,j)+p(i,j))/abs(p(i+2,j)+2.0*p(i+1,j)
     $ +p(i,j)))

       eps4(i,j)=max(0.0,(kp4-eps2(i,j)))
       End Forall

      Forall(k=1:lmax,i=2:mx-1,j=2:my)
      avbc(k,i,j)=
     $ 0.5*
     $ (V(2,i,j)/V(1,i,j)+V(2,i+1,j)/V(1,i+1,j))*dnbx(i,j)

    	 avbc(k,i,j)=avbc(k,i,j)+
     $ 0.5*(V(3,i,j)/V(1,i,j)+V(3,i+1,j)/V(1,i+1,j))*dnby(i,j)
    	avbc(k,i,j)=abs(avbc(k,i,j))

    	avbc(k,i,j)=avbc(k,i,j)
     $ /sqrt(dnbx(i,j)**2+dnby(i,j)**2) 
    	avbc(k,i,j)=avbc(k,i,j)+ sqrt(1.4*(p(i,j)
     $ + p(i+1,j))/(V(1,i,j)+V(1,i+1,j)))

    	avbc(k,i,j)=avbc(k,i,j)*(eps2(i,j)*(V(k,i+1,j)
     $ -V(k,i,j)) -eps4(i,j)*(V(k,i+2,j)-V(k,i+1,j)-V(k,i+1,j)
     $ - V(k,i+1,j)+V(k,i,j)+V(k,i,j)+V(k,i,j)
     $ -V(k,i-1,j)))
      End Forall
    	
    	
      Forall(k=1:lmax,i=3:mx,j=2:my) avda(k,i,j)=1.0*avbc(k,i-1,j)
    		

    	End subroutine

